'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.default = function (_ref) {
  var types = _ref.types;

  var t = types;

  // TODO: Fix Babylon allowing namespaced member expression

  return {
    inherits: require('babel-plugin-syntax-jsx'),
    visitor: {
      JSXElement: visitJSXElement
    }
  };

  function visitJSXElement(path) {
    var node = path.node;


    path.replaceWith(isComponentName(node.openingElement.name) ? toComponent(node) : toDomElement(node));

    function toDomElement(_ref2) {
      var _ref2$openingElement = _ref2.openingElement;
      var jsxAttributes = _ref2$openingElement.attributes;
      var name = _ref2$openingElement.name;
      var children = _ref2.children;

      var _ref3 = isNamespacedName(name) ? [name.name.name, name.namespace.name] : [name.name, undefined];

      var _ref4 = _slicedToArray(_ref3, 2);

      var elementName = _ref4[0];
      var namespaceName = _ref4[1];


      var objectProperties = [];

      if (namespaceName !== undefined) {
        objectProperties.push(objectProperty('namespaceName', t.stringLiteral(namespaceName)));
      }

      objectProperties.push(objectProperty('attributes', toAttributesArray(jsxAttributes)), objectProperty('children', mapChildren(children)));

      return streamDomCallExpression('element', [t.stringLiteral(elementName), t.objectExpression(objectProperties)]);
    }

    function toComponent(_ref5) {
      var _ref5$openingElement = _ref5.openingElement;
      var jsxAttributes = _ref5$openingElement.attributes;
      var name = _ref5$openingElement.name;
      var children = _ref5.children;

      return streamDomCallExpression('component', [isJsxMemberExpression(name) ? toMemberExpression(name) : t.identifier(name.name), t.objectExpression([objectProperty('attributes', toAttributesArray(jsxAttributes)), objectProperty('children', mapChildren(children))])]);
    }

    function toAttributesArray(jsxAttributes) {
      return t.arrayExpression(jsxAttributes.map(function (jsxAttribute) {
        return jsxAttribute.type === 'JSXSpreadAttribute' ? jsxAttribute.argument : toAttributeObject(jsxAttribute);
      }));
    }

    function toAttributeObject(jsxAttribute) {
      var nameNode = jsxAttribute.name;
      var valueNode = jsxAttribute.value;

      var _ref6 = isNamespacedName(nameNode) ? [nameNode.namespace.name, nameNode.name] : [null, nameNode];

      var _ref7 = _slicedToArray(_ref6, 2);

      var namespace = _ref7[0];
      var jsxIdentifier = _ref7[1];


      var attributeProperties = [];

      if (namespace) {
        attributeProperties.push(objectProperty('namespace', t.stringLiteral(namespace)));
      }

      attributeProperties.push(objectProperty('name', t.stringLiteral(jsxIdentifier.name)), objectProperty('value', valueNode === null ? t.booleanLiteral(true) : valueNode.type === 'JSXExpressionContainer' ? valueNode.expression : valueNode));

      return t.objectExpression(attributeProperties);
    }

    function mapChildren(children) {
      return t.arrayExpression(children.map(function (childNode) {
        return childNode.type === 'JSXText' ? toText(childNode.value) : childNode.type === 'JSXExpressionContainer' ? toExpression(childNode.expression) : childNode;
      }));
    }

    function toText(str) {
      return streamDomCallExpression('text', [t.stringLiteral(str)]);
    }

    function toExpression(expressionNode) {
      return streamDomCallExpression('expression', [expressionNode]);
    }

    function isComponentName(name) {
      return !isNamespacedName(name) && (isJsxMemberExpression(name) || isJsxIdentifier(name) && /^[A-Z]/.test(name.name));
    }

    function toMemberExpression(_ref8) {
      var object = _ref8.object;
      var property = _ref8.property;

      return t.memberExpression(isJsxMemberExpression(object) ? toMemberExpression(object) : t.identifier(object.name), t.identifier(property.name));
    }

    function isJsxIdentifier(name) {
      return name.type === 'JSXIdentifier';
    }

    function isJsxMemberExpression(name) {
      return name.type === 'JSXMemberExpression';
    }

    function isNamespacedName(name) {
      return name.type === 'JSXNamespacedName';
    }

    function streamDomCallExpression(name, args) {
      return t.callExpression(t.memberExpression(t.identifier('streamDom'), t.identifier(name)), args);
    }

    function objectProperty(key, value) {
      return t.objectProperty(toObjectKeyType(key), value);
    }

    function toObjectKeyType(key) {
      return _esutils2.default.keyword.isIdentifierNameES6(key) ? t.identifier(key) : t.stringLiteral(key);
    }
  }
};

var _esutils = require('esutils');

var _esutils2 = _interopRequireDefault(_esutils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }